// Copyright (c) 2024 Celestino Amoroso (celestino.amoroso@gmail.com).
// All rights reserved.

// context.go
package expr

// ---- Function template
type FuncTemplate func(ctx exprContext, name string, args []any) (result any, err error)

// ---- Functor interface
type Functor interface {
	Invoke(ctx exprContext, name string, args []any) (result any, err error)
}

type simpleFunctor struct {
	f FuncTemplate
}

func (functor *simpleFunctor) Invoke(ctx exprContext, name string, args []any) (result any, err error) {
	return functor.f(ctx, name, args)
}

// ---- Function Info
type exprFunc interface {
	Name() string
	MinArgs() int
	MaxArgs() int
	Functor() Functor
}

// ----Expression Context
type exprContext interface {
	Clone() exprContext
	GetVar(varName string) (value any, exists bool)
	SetVar(varName string, value any)
	EnumVars(func(name string) (accept bool)) (varNames []string)
	EnumFuncs(func(name string) (accept bool)) (funcNames []string)
	GetFuncInfo(name string) exprFunc
	Call(name string, args []any) (result any, err error)
	RegisterFunc(name string, f Functor, minArgs, maxArgs int)
}
